Struct IterMut

struct IterMut<'a, B: 'a + BitBlock = u32> { ... }

An iterator for mutable references to the bits in a BitVec.

Implementations

impl<'a, B = u32> RefUnwindSafe for IterMut<'a, B>

impl<'a, B = u32> Send for IterMut<'a, B>

impl<'a, B = u32> Sync for IterMut<'a, B>

impl<'a, B = u32> UnwindSafe for IterMut<'a, B>

impl<'a, B> Freeze for IterMut<'a, B>

impl<'a, B> Unpin for IterMut<'a, B>

impl<'a, B> UnsafeUnpin for IterMut<'a, B>

impl<'a, B: BitBlock> DoubleEndedIterator for IterMut<'a, B>

fn next_back(self: &mut Self) -> Option<<Self as >::Item>

impl<'a, B: BitBlock> ExactSizeIterator for IterMut<'a, B>

impl<'a, B: BitBlock> Iterator for IterMut<'a, B>

fn next(self: &mut Self) -> Option<<Self as >::Item>
fn size_hint(self: &Self) -> (usize, Option<usize>)

impl<I> IntoIterator for IterMut<'a, B>

fn into_iter(self: Self) -> I

impl<T> Any for IterMut<'a, B>

fn type_id(self: &Self) -> TypeId

impl<T> Borrow for IterMut<'a, B>

fn borrow(self: &Self) -> &T

impl<T> BorrowMut for IterMut<'a, B>

fn borrow_mut(self: &mut Self) -> &mut T

impl<T> From for IterMut<'a, B>

fn from(t: T) -> T

Returns the argument unchanged.

impl<T, U> Into for IterMut<'a, B>

fn into(self: Self) -> U

Calls U::from(self).

That is, this conversion is whatever the implementation of [From]<T> for U chooses to do.

impl<T, U> TryFrom for IterMut<'a, B>

fn try_from(value: U) -> Result<T, <T as TryFrom<U>>::Error>

impl<T, U> TryInto for IterMut<'a, B>

fn try_into(self: Self) -> Result<U, <U as TryFrom<T>>::Error>